breeds [cells]

cells-own [
  neighbors6   ;; agentset of 6 neighboring cells
  n            ;; used to store a count of white neighbors
]
to setup1
  ca
  setup-grid
  ;; make the center hexagon white, as the "seed" from which we
  ;; grow our "snowflake"
  ask patch 0 0
    [ ask cells-here
        [ set color white ] ]
end

to setup2
  ca
  setup-grid
  ;; make two center hexagons white, as the "seed" from which we
  ;; grow our "snowflake"
  ask patches at-points [[0 0] [1 0]]
    [ ask cells-here
        [ set color white ] ]
end

to setup-grid
  set-default-shape cells "hex"
  ask patches
    [ ;; if we don't want wrapping around the edge of the screen, the easiest
      ;; way to accomplish that is simply to leave the outer edge blank
      if wrap? or (abs pxcor < screen-edge-x and abs pycor < screen-edge-y)
        [ sprout 1
            [ set breed cells
              set color gray - 3  ;; dark gray
              ;; shift even columns down
              if pxcor mod 2 = 0
                [ set ycor ycor - 0.5 ] ] ] ]
  ;; now set up the neighbors6 agentsets
  ask cells
    [ ifelse pxcor mod 2 = 0
        [ set neighbors6 cells-on patches at-points [[0 1] [1 0] [1 -1] [0 -1] [-1 -1] [-1 0]] ]
        [ set neighbors6 cells-on patches at-points [[0 1] [1 1] [1  0] [0 -1] [-1  0] [-1 1]] ] ]
end

to go
  ;; these are example cellular automaton rules; they produce
  ;; a snowflake-like pattern
  ask cells
    [ set n count neighbors6 with [color = white] ]
  ask cells
    [ if n = 1
        [ set color white ] ]
end
@#$#@#$#@
GRAPHICS-WINDOW
240
10
777
568
15
15
17.0
1
10
1
1
1

CC-WINDOW
3
271
238
568
Command Center

BUTTON
32
44
113
77
NIL
setup1
NIL
1
T
OBSERVER
T

BUTTON
73
114
154
147
NIL
go
T
1
T
OBSERVER
T

BUTTON
73
79
154
112
step
go
NIL
1
T
OBSERVER
T

SWITCH
55
180
168
213
wrap?
wrap?
0
1
-1000

BUTTON
115
44
189
77
NIL
setup2
NIL
1
T
OBSERVER
T

@#$#@#$#@
WHAT IS IT?
-----------
This demonstrates how to make a model that uses a hexagonal grid of cells.

The cellular automaton rules used produce a snowflake-like pattern.


HOW IT WORKS
------------
Since patches are square, we must represent the cells as turtles instead.

Each patch sprouts a "cell" turtle.  Turtles on even patch columns are offset down by half a patch.  (Since the south boundary of a patch is part of the patch, this does not move the turtle to a different patch.)

The resulting lattice has the correct structure, but distances and angles are distorted.  So the hexagons aren't actually regular, but it doesn't matter as long as you don't try to use primitives such as DISTANCE and TOWARDS.

Also, you must use the NEIGHBORS6 variable instead of the built-in NEIGHBORS and NEIGHBORS4 reporters.


CREDITS AND REFERENCES
----------------------
This example is for stationary cells.  "Hex Turtles Example" shows how to make turtles that move along a hexagonal lattice.  (It would also be possible to combine both techniques in a single model.)
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

hex
false
0
Polygon -7566196 true true 0 150 75 30 225 30 300 150 225 270 75 270

@#$#@#$#@
NetLogo 2.0.0
@#$#@#$#@
setup1
repeat 20 [ go ]
@#$#@#$#@
@#$#@#$#@
