globals 
[
  fast average slow stay-put      ;; current counts
  avg-speed avg-kinetic-energy    ;; current averages
  mean-wall-collisions            ;; mean number of collisions per particle with wall
  mean-particle-collisions        ;; mean number of collisions per particle with other particles
  avg-speed-init avg-energy-init  ;; initial averages
  clock vsplit vclock             ;; clock variables
  box-edge                        ;; patch coords of box's edge
  pressure                        ;; pressure
  pressure-history                ;; history of last pressure-hist-size
                                  ;; pressures
  pressure-hist-size              ;; max length of pressure-history
]

turtles-own 
[
  speed  kinetic-energy  mass          ;; turtle info
  wall-collisions
  wall-collisions-history
  particle-collisions
  particle-collisions-history
  v1t v1l tmp-turtle                    ;; collision info (turtle 1)
  heading2 mass2 speed2 v2t v2l turtle2 ;; collision info (turtle 2)
  theta                                 ;; collision info (both turtles)
  momentum                              ;; particles momentum at the moment
                                        ;; when it strikes the wall.
]

to setup
  locals [ my-patch patches-in-box ]
  ca
  set pressure-hist-size 10
  set pressure-history []
  set box-edge (round (screen-edge-x * box-size / 100))
  set patches-in-box ((box-edge - 1) * (box-edge - 1) * 4 + 4 * box-edge + 1 )
  if box? [ make-box ]
  set clock 0
  set vclock 0
  set mean-wall-collisions 0
  set mean-particle-collisions 0
  cct number 
  [ set speed 10
    set mass particle-mass
    set momentum 0
    set wall-collisions-history [0]
    set particle-collisions-history [0]
    if one-dimension-motion? [ set heading 90 + random-int-or-float 2 * 180 ]
    ifelse ( box? and particles-in-box?) 
    [
      random-position 
    ] 
    [
      setxy (random-int-or-float screen-size-x) (random-int-or-float screen-size-y)
    ] 
    set shape "circle"
    recolor
  ]
  update-variables
  set avg-speed-init avg-speed
  set avg-energy-init avg-kinetic-energy
  setup-plots
  do-plotting
end

to update-variables
  ask turtles
    [ set kinetic-energy (0.5 * speed * speed * mass) ]
  set average count turtles with [color = green]
  set slow    count turtles with [color = blue]
  set fast    count turtles with [color = red]
  set stay-put count turtles with [color = white]
  set avg-speed  mean values-from turtles [speed]
  set avg-kinetic-energy mean values-from turtles [kinetic-energy]
  set vsplit ceiling max values-from turtles [speed]
  set mean-wall-collisions (mean values-from turtles [mean wall-collisions-history])
  set mean-particle-collisions (mean values-from turtles [mean particle-collisions-history])
  calculate-pressure
end

to calculate-pressure
  set pressure sum values-from turtles [momentum]
  ifelse length pressure-history < pressure-hist-size
  [ set pressure-history fput pressure pressure-history ]
  [ set pressure-history fput pressure (but-last pressure-history) ]
end

to recalculate-vsplit
  ;; this needs to be done without-interruption to be sure that nothing tries to
  ;; use vsplit or vclock before they've been recalculated
  without-interruption
  [ set vsplit 2 * vsplit
    set vclock 2 * vclock
  ]
end

to go
  if bounce? [ ask turtles [ bounce ]]
  without-interruption 
  [
    ask turtles [ move ]
  ]
  set vclock (vclock + 1)
  if (vclock = vsplit)
  [ set clock (clock + 1)
    set vclock 0
    update-variables
    do-plotting
    ask turtles [ set momentum 0 ]
  ]
end

to bounce  ;; turtle procedure
  locals [new-px new-py]
  ; if we're not about to hit a wall (yellow patch),
  ; we don't need to do any further checks
  if pcolor-of patch-at dx dy != yellow 
  [ 
    set wall-collisions 0
    ifelse length wall-collisions-history < 10
      [ set wall-collisions-history fput wall-collisions wall-collisions-history ]
      [ set wall-collisions-history fput wall-collisions (but-last wall-collisions-history) ]
    stop 
  ]
  ; get the coordinates of the patch we'll be on if we go forward 1
  set new-px round (xcor + dx)
  set new-py round (ycor + dy)
  set wall-collisions 1
  ifelse length wall-collisions-history < 10
    [ set wall-collisions-history fput wall-collisions wall-collisions-history ]
    [ set wall-collisions-history fput wall-collisions (but-last wall-collisions-history) ]
  ; check: hitting left or right wall?
  if (abs new-px = box-edge)
    ; if so, reflect heading around x axis
    [ set heading (- heading)
      if (abs pxcor < box-edge)
        [ set momentum momentum + abs (sin heading * mass * speed) ]
    ]
  ; check: hitting top or bottom wall?
  if (abs new-py = box-edge)
    ; if so, reflect heading around y axis
    [ set heading (180 - heading)
      if (abs pycor < box-edge)
        [ set momentum momentum + abs (cos heading * mass * speed) ]
    ]
end

to move-to-unoccupied-patch ;; two particles cannot occupy the same space
  locals [ empty-patch new-x new-y ]
  without-interruption
  [
    while [(speed / vsplit) >= 1.0]
      [ recalculate-vsplit ]   
    set new-x ((speed / vsplit) * dx)
    set new-y ((speed / vsplit) * dy)
    if (pcolor-of (patch-at new-x new-y) = yellow ) and (bounce?) 
    [
      ;bounce
      stop
    ]
    ifelse count (turtles-at new-x new-y) with [ who != who-of myself ] = 0
    [
      jump (speed / vsplit)
      set particle-collisions 0
      ifelse length particle-collisions-history < 10
        [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
        [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history)]
    ]
    [
      ifelse collisions?
      [ set tmp-turtle random-one-of (turtles-at new-x new-y) with [who != who-of myself]
    ;    if (who > who-of tmp-turtle)
    ;      [ collide ]
          collide
          set particle-collisions 1
          ifelse length particle-collisions-history < 10
            [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
            [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history)]
      ]
      [  set empty-patch random-one-of neighbors with [ not any? turtles-here and pcolor = black ] in-radius (speed / vsplit)
         if empty-patch != nobody
          [ 
            set heading towards empty-patch
            jump (speed / vsplit)
            set particle-collisions 0
            ifelse length particle-collisions-history < 10
              [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
              [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history)]
          ]          
      ]
     ]
   ]
end


to move  ;; turtle procedure
  locals [ new-x new-y ]
  while [(speed / vsplit) >= 1.0]
  [ recalculate-vsplit ]
  set new-x ((speed / vsplit) * dx)
  set new-y ((speed / vsplit) * dy)
  if (pcolor-of (patch-at new-x new-y) = yellow ) and (bounce?) 
  [
    ;bounce
    stop
  ]
  jump (speed / vsplit)
  if collisions? 
  [ ifelse not one-dimension-motion? 
    [ check-for-collision ] 
    [ check-for-collision-one-dimension ]
  ]
end

to check-for-collision  ;; turtle procedure
  ifelse count other-turtles-here >= 1
    [ set tmp-turtle random-one-of other-turtles-here
      ;if ((who > who-of tmp-turtle) and (turtle2 != tmp-turtle))
      ;  [ 
          collide 
          set particle-collisions 1
          ifelse length particle-collisions-history < 10
            [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
            [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
      ;  ]
    ]
    [
      set particle-collisions 0
      ifelse length particle-collisions-history < 10
        [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
        [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
    ]
end
        
to check-for-collision-one-dimension  ;; turtle procedure
  ifelse count other-turtles-here with [ heading != heading-of myself ] >= 1
    [ set tmp-turtle random-one-of other-turtles-here with [ heading != heading-of myself ]
      if ((who > who-of tmp-turtle) and (turtle2 != tmp-turtle))
        [ 
          collide
          set particle-collisions 1
          ifelse length particle-collisions-history < 10
            [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
            [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
        ]
    ]
    [
      set particle-collisions 0
      ifelse length particle-collisions-history < 10
        [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
        [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
    ]
end

to collide  ;; turtle procedure
  get-turtle2-info
  if not one-dimension-motion? [ calculate-velocity-components ]
  set-new-speed-and-headings
end

to get-turtle2-info  ;; turtle procedure
  set turtle2 tmp-turtle
  set mass2 mass-of turtle2
  set speed2 speed-of turtle2
  set heading2 heading-of turtle2
end

to calculate-velocity-components
  locals [vcm]  ;; CM vel. along dir. theta
  set theta (random-int-or-float 360)
  set v1l (speed * sin (theta - heading))
  set v1t (speed * cos (theta - heading))
  set v2l (speed2 * sin (theta - heading2))
  set v2t (speed2 * cos (theta - heading2))
  set vcm (((mass * v1t) + (mass2 * v2t)) / (mass + mass2))
  set v1t (vcm + vcm - v1t)
  set v2t (vcm + vcm - v2t)
end

to set-new-speed-and-headings  ;; turtle procedure
  locals [ temp-speed ]
  set temp-speed speed
  ifelse one-dimension-motion? 
  [ set speed speed2 ]
  [ set speed sqrt ((v1t * v1t) + (v1l * v1l)) ]
  ifelse one-dimension-motion? 
      [ set heading heading + 180 ]
      [ set heading (theta - (atan v1l v1t)) ]
  ifelse one-dimension-motion?
      [ set speed-of turtle2 temp-speed ]
      [ set speed-of turtle2 sqrt ((v2t * v2t) + (v2l * v2l)) ]
  ifelse one-dimension-motion?
      [ set heading-of turtle2 heading-of turtle2 + 180 ]
      [ set heading-of turtle2 (theta - (atan v2l v2t)) ]
  recolor
  ask turtle2 
    [ 
      recolor
      set particle-collisions 1
      ifelse length particle-collisions-history < 10
        [ set particle-collisions-history fput particle-collisions particle-collisions-history ]
        [ set particle-collisions-history fput particle-collisions (but-last particle-collisions-history) ]
    ]
end

to recolor  ;; turtle procedure
  ifelse ( speed < (0.5 * 10) and speed > 0 )
    [ set color blue ]
    [ ifelse speed > (1.5 * 10)
        [ set color red ]
        [ set color green ] ]
end

to make-box
  ask patches with [((abs pxcor = box-edge) and (abs pycor <= box-edge)) or
                    ((abs pycor = box-edge) and (abs pxcor <= box-edge))]
    [ set pcolor yellow ]
end

to random-position ;; turtle procedure
  setxy ((1 - box-edge) + random-int-or-float (2 * box-edge - 1))
        ((1 - box-edge) + random-int-or-float (2 * box-edge - 1))
end

;;; plotting procedures
to setup-plots
  set-current-plot "Speed Counts"
  set-plot-y-range 0 number
end

to do-plotting
  set-current-plot "Speed Counts"
  set-current-plot-pen "fast"
  plot fast
  set-current-plot-pen "medium"
  plot average
  set-current-plot-pen "slow"
  plot slow

  set-current-plot "collisions"
  set-current-plot-pen "wall collisions"
  plot mean-wall-collisions  
  set-current-plot-pen "particle collisions"
  plot mean-particle-collisions
  
  set-current-plot "Average speed"
  plot avg-speed
end



; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo GasLab Gas in a Box model.
; http://ccl.northwestern.edu/netlogo/models/GasLabGasinaBox.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/GasLabGasinaBox
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
280
12
645
398
35
35
5.0
1
10
1
1
1

CC-WINDOW
677
431
925
614
Command Center

MONITOR
595
508
676
557
clock
clock
0
1

MONITOR
341
581
422
630
fast
fast
0
1

MONITOR
506
580
587
629
slow
slow
0
1

SLIDER
10
212
223
245
number
number
1
500
200
1
1
NIL

BUTTON
115
14
190
47
go
go
T
1
T
OBSERVER
T

BUTTON
31
14
106
47
NIL
setup
NIL
1
T
OBSERVER
T

TEXTBOX
11
73
156
102
Objects: Particles and...

TEXTBOX
9
323
132
342
Interactions

MONITOR
423
580
504
629
medium
average
3
1

PLOT
335
413
594
580
Speed counts
Time
Count
0.0
100.0
0.0
100.0
true
false
PENS
"medium" 1.0 0 -11352576 true
"slow" 1.0 0 -16776961 true
"fast" 1.0 0 -65536 true

MONITOR
212
579
314
628
average speed
avg-speed
3
1

PLOT
64
413
329
579
Average speed
Time
Speed
0.0
100.0
0.0
12.0
true
false
PENS
"speed" 1.0 0 -7566196 true

MONITOR
69
579
211
628
average kinetic energy
avg-kinetic-energy
3
1

SLIDER
9
138
223
171
box-size
box-size
0
100
75
1
1
%

SWITCH
9
174
161
207
particles-in-box?
particles-in-box?
0
1
-1000

SWITCH
165
67
269
100
box?
box?
1
1
-1000

SWITCH
13
344
123
377
collisions?
collisions?
1
1
-1000

TEXTBOX
15
114
134
133
Properties

SLIDER
10
248
223
281
particle-mass
particle-mass
2
50
2
1
1
a.m.u.

SWITCH
126
344
230
377
bounce?
bounce?
1
1
-1000

SWITCH
10
287
223
320
one-dimension-motion?
one-dimension-motion?
1
1
-1000

PLOT
661
200
930
426
collisions
time
collisions
0.0
100.0
0.0
0.01
true
true
PENS
"wall collisions" 1.0 0 -44544 true
"particle collisions" 1.0 0 -65413 true

MONITOR
676
138
771
187
 wall collisions
mean-wall-collisions
3
1

MONITOR
784
139
898
188
 particle collisions
mean-particle-collisions
3
1

TEXTBOX
672
100
920
119
Number of collisions per particle per time tick

MONITOR
693
44
811
93
   pressure in box
pressure
0
1

@#$#@#$#@
Connected Chemistry Curriculum is currently under development, for more information please refer to http://ccl.northwestern.edu/curriculum/chemistry/


WHAT IS IT?
-----------
This model simulates the behavior of gas particles in a changeable container.  The Kinetic Molecular Theory (KMT) underlies the rules that the particles follow, and some of these rules can be added or taken away from the model.  Whether particles are contained in a box, bounce off its walls or collide with each other can be determined.  These rules can then be related to the particles' speed distributions and their rate of collisions with the container walls and among themselves.  Furthermore, these features can then be compared with the pressure inside the container.

This model is part of the Connected Chemistry High School Curriculum and connects students prior learning about pressure with the Kinetic Molecular Theory.  This is one of a number of models that have been adapted from the model Gas in a Box, which can be found in the GasLab folder under Chemistry and Physics in the models library.  

In all of the Connected Chemistry Curriculum models, the same basic rules are used for expressing what happens when gas particles collide.  Each model has different features in order to show different aspects of the behavior of gases.


HOW IT WORKS
------------
Particles have kinetic energy (that which is due to their motion).  If the particles collide among themselves, these collisions are perfectly elastic.  Particles are colored according to their speed -- blue for slow, green for medium, and red for high speeds.

When choosing to have the particles collide, they do so, conserving energy and momentum, in the following way:
1. Two particles "collide" if they find themselves on the same patch.
2. A random axis is chosen, as if they are two balls that hit each other and this axis is the line connecting their centers.
3. They exchange momentum and energy along that axis, according to the conservation of momentum and energy.  This calculation is done in the center of mass system.
4. Each turtle is assigned its new velocity, energy, and heading.

Choosing that the model will have a container, and deciding to have the particles bounce off the walls means that these collisions are elastic, or that if a turtle finds itself on or very close to a wall of the container, it "bounces" -- that is, reflects its direction and keeps its same speed.


HOW TO USE IT
-------------
Initial settings:
Select the objects that the model will include: Particles and...
- BOX? selects whether the model will include a box.
Select the properties of the objects in the model.
For the box:
- BOX-SIZE: size of the box. (percentage of the graphics screen size)
For the particles:
- PARTICLES-IN-BOX?: selects whether the particles will start out inside the box.
- NUMBER: number of gas particles
- PARTICLE-MASS: mass of the particles in a.m.u. (atomic mass units)
- ONE-DIMENSIONAL-MOTION?: selects whether the particles will move in one dimension or two dimensions.
Select the interactions among the objects in the model:
- BOUNCE?: selects whether the particles will bounce off the box walls or not.
- COLLISIONS?: determines if the particles will collide among themselves.
Some of these settings can be changed while the model is running.

The SETUP button will set the initial conditions.  The GO button will run the simulation.

Monitors:
- FAST, MEDIUM, SLOW: numbers of particles with different speeds: fast (red), medium (green), and slow (blue).
- CLOCK: number of ticks that have run.
- AVERAGE-SPEED: average speed of the particles.
- AVERAGE-KINETIC-ENERGY: average kinetic energy of the particles.
- WALL-COLLISIONS: average number of collisions with the wall per particle per time-tick (collision rate per particle)
- PARTICLE-COLLISIONS: average number of collisions among particles, per particle per time-tick (collision rate per particle)
- PRESSURE: the total pressure inside  the box

Plots:
- AVERAGE SPEED: plots the average particle speed
- SPEED COUNTS: plots the number of particles in each range of speed (slow, medium faste).
- COLLISIONS: plots the average rate of collisions, for particles' collisions with the wall, and for particles' collisions among themselves.

Initially, all the particles have the same speed but random directions in either one dimension or two dimensions.  If so chosen, the particles repeatedly collide, they exchange energy and head off in new directions, and the speeds are dispersed -- some particles get faster, some get slower.


THINGS TO NOTICE
----------------
What are the necessary conditions for the particles to remain in the box while the model is running?  

When do the particles change their speed?  

Can you observe collisions and color changes as they happen?  For instance, when a red particle hits a green particle, what color do they each become?  For this purpose, you can set the graphics screen to include turtle sizes and their exact positions.  Then, you can zoom in and observe the particles closely.  

How does the average speed change over time?  Is energy conserved in this system?
How is the average speed related to the speed counts?

Observe the collisions plot when you increase the number of particles: do both wall collisions and particle collisions change in the same way?  How is this related to pressure?

What changes when the mass of the particles is greater?  What doesn't change?

Does the pressure change when the particles don't collide with each other in the model?

Does the pressure change when the model is changed to operate in only one dimension?


THINGS TO TRY
-------------
What happens if there is only one particle in the box?  Very many particles in the box?  How do their collision rates change?

Create a monitor for the speed of one particle.  Compare its speed as the model runs with the average speed.  Can you connect the pattern you have found with the Particle count plot?

Does the number of dimensions in which the particles can move make a difference?  Test different properties in the model quantitatively and see whether there are consistent patterns when comparing the particles moving through one or two dimensions.

Create two monitors counting collisions for one particle -- one with the wall, one with other particles.  Run the model to see the relationship between the average values and those of a single particle.

Test the different settings to examine which one of them changes the pressure.  Can you determine the way pressure is measured in this model?

Observe a small number of particles when the number of particles is very large (a thousand or more).  You can hide the turtles by typing in "ask turtles [ ht ]" in the command center, and then add in some turtles by typing "ask turtle 3 [ st ]".   How is the single particle movement related to the patterns you have found for the group?

EXTENDING THE MODEL
-------------------
What would happen if particles took up space?  You can try and have them occupy and occlude a patch.  Would the pressure change?  Why?

What happens if the collisions are non-elastic?

How does the 2-D model differ from a 3-D model?  How does it differ from the 1-D model?

Find a way to monitor how far particles go between collisions, on the average.  The latter is called the "mean free path".  What factors affect its value?


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (2002).  NetLogo Chem KMT model.  http://ccl.northwestern.edu/netlogo/models/ChemKMT. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2002 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/ChemKMT for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

circle
false
0
Circle -7566196 true true 35 35 230

@#$#@#$#@
NetLogo 2.0beta5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
