globals [
  iteration         ;; how many times we've iterated so far
]

;; We use the built-in turtle variable SIZE to make the line
;; segment have the appropriate length.  Because the "segment"
;; shape extends from the center of the turtle to its edge,
;; rather than from edge to edge, we need to set the size
;; to twice the segment length in order for the turtles to
;; appear the right size.  This is why we multiple or divide
;; by two in several places in the following code.

to setup 
  ca
  set-default-shape turtles "segment"
  set iteration 0
  create-custom-turtles 1
    [ setxy (- screen-edge-x)       ;; start turtle in lower left corner
            (- screen-edge-y)
      set heading 90                ;; facing right
      set color blue
      set size screen-size-x * 2 ]
  update-plot
end

to step 
  ask turtles
    [ iterate ]
  set iteration iteration + 1
  update-plot
end

to iterate
  set size size / 3
  hatch 1 [ ]
  fd size / 2
  lt 60
  hatch 1 [ ]
  fd size / 2
  rt 120
  hatch 1 [ ]
  fd size / 2
  lt 60
end

to update-plot
  plot sum values-from turtles [size / 2]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2002.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Koch Curve model.
; http://ccl.northwestern.edu/netlogo/models/KochCurve.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 1998 by Uri Wilensky.  All rights reserved.  See
; http://ccl.northwestern.edu/netlogo/models/KochCurve
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
4
63
500
276
121
45
2.0
1
10
1
1
1

CC-WINDOW
229
284
565
466
Command Center

BUTTON
13
18
94
51
Set Up
setup
NIL
1
T
OBSERVER
T

MONITOR
229
10
292
59
Iteration
iteration
3
1

BUTTON
13
18
94
51
Set Up
setup
NIL
1
T
OBSERVER
T

MONITOR
229
10
292
59
Iteration
iteration
3
1

BUTTON
13
18
94
51
Set Up
setup
NIL
1
T
OBSERVER
T

MONITOR
229
10
292
59
Iteration
iteration
3
1

BUTTON
13
18
94
51
setup
setup
NIL
1
T
OBSERVER
T

BUTTON
101
18
183
51
NIL
step
NIL
1
T
OBSERVER
T

MONITOR
229
10
292
59
NIL
iteration
3
1

MONITOR
292
10
383
59
total length
sum values-from turtles\n[size / 2]
5
1

PLOT
2
284
227
466
Total Length
NIL
NIL
0.0
5.0
0.0
1000.0
true
false

MONITOR
386
10
492
59
# segments
count turtles
0
1

@#$#@#$#@
WHAT IS IT?
------------
Helge von Koch was a Swedish mathematician who, in 1904, introduced what is now called the Koch curve.

Here is a simple geometric construction of the Koch curve.  Begin with a straight line.  This initial object is also called the "initiator."  Partition it into three equal parts.  Then replace the middle third by an equilateral triangle and take away its base.  This completes the basic construction step.  A reduction of this figure, made of four parts, will be used in the following stages.  It is called the "generator."  Thus, we now repeat, taking each of the resulting line segments and partitioning them into three equal parts, and so on.  The figure below illustrates this iterative process.


|     ________________________     Step 0: "Initiator"
|  
|                /\  
|               /  \ 
|              /    \ 
|             /      \
|     _______/        \_______     Step 1: "Generator" 
|  
|                /\  
|             __/  \__ 
|             \      /
|             /      \   
|     ___/\__/        \__/\___     Step 2

Self-similarity is built into the construction process.  Each part of the four parts in the k-th step is again a version scaled down by the factor of 3 of the entire curve in the previous (k-1)-st step. 

The Koch curve is as difficult to understand as the Cantor set or the Sierpinski tree.  However the problems with it are of a different nature.  First of all -- as the name already expresses -- it is a curve, but this is not clear from the construction.  Secondly, this curve contains no straight lines which are smooth in the sense that we could see them as a carefully bent line.   Rather this curve has much of the complexity which we could see in a natural coastline: folds within folds within folds and so on.

Actually Koch's motivation for finding this curve was to provide another example for the discovery made by the German mathematician Karl Weierstrass, who in 1872 had precipitated a minor crisis in mathematics.  He had described a curve that could not be differentiated (did not have a tangent) at any of its points.  The ability to differentiate is central to differentable calculus and for a long time it was assumed that curves have tangent lines almost everywhere.

Let us now discuss the length of the Koch curve.  After the first iteration we have a curve which is made of four line segments of the same length, after the second iteration we will have each of the four segments broken into four more segments i.e. sixteen segments and so on.  After each iteration we increase the number of segments by the factor of four.  If we denote the number of segments after k-steps by S(k) then mathematically:
                                 
|             k   
|     S(k) = 4

Now if the initial segment had length L the length of each of the four segments obtained after the first stage would be L/3.  After the second step the length of each of the sixteen segments is (L/3)/3 or L/9.  Denoting the length of each segment during the k-th iteration by L(k) we may write:

|                k  
|     L(k) = L/(3 )

Multiplying the number of segments by the length of each segment we get the following expression for the length of the Koch curve after k steps of construction:

|            k
|     L*(4/3) 


Clearly the length grows exponentially with the number of iterations so in fact the length of the entire Koch curve is infinite as is the arc length between any of its two points.  It therefore might come as a surprise that the area enclosed by the Koch curve is finite; the proof of this we leave as an exercise for the reader. 


HOW TO USE IT
-------------
Reset the program by pushing the SETUP button.  This will clear the graphics screen, create the initiator and initialize the globals.  Press repeatedly on the STEP button.  Each time you press this button the construction algorithm is iterated and you will see successive approximations of the Koch curve.


THINGS TO NOTICE
----------------
What happens to the total length of the curve as the iteration progresses?

To remove the middle third during each stage we simply change the color of the turtle to black in the "move" procedure.  Then the turtle erases as it moves along the middle third.

The turtles are drawn in red so you can more easily visualize how the curve is constructed.

When the middle sections of the segments are erased with black, sometimes the erasure isn't perfect and a few junk blue patches are left behind.  This is because when a turtle's pen is down, it colors only the patches it lands on, not necessarily every patch it crosses, as it jumps one patch at a time.  Therefore the black turtle doing the erasing won't necessarily land on the exact same patches as the blue turtle that did the drawing, since the two turtles didn't begin from the same position.


THINGS TO TRY
-------------
Try running the model through several iterations. Can you see how the recursive design is changing from one iteration to another? Note that each successive iteration takes longer to compute. Depending on the speed of your machine, high-numbered iterations could take a long time!


EXTENDING THE MODEL
--------------------
You can combine three copies of the Koch curve to form a closed curve called the Koch snowflake. Try to write a program that draws this curve. 

Can you think of other initiators and generators? Try and implement a few. Can you characterize which initiators and generators lead to "interesting shapes"?


NETLOGO FEATURES
----------------
Notice how the curves are made out of many turtles, all following the same rules.  Also, take note of the use of the HATCH command to create all of the turtles by repeated "cloning" from a single seed turtle.


CREDITS AND REFERENCES
----------------------
To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Koch Curve model. http://ccl.northwestern.edu/netlogo/models/KochCurve. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 1998 by Uri Wilensky.  All rights reserved.  See http://ccl.northwestern.edu/netlogo/models/KochCurve for terms of use.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

segment
true
0
Circle -65536 true false 135 135 30
Line -7566196 true 150 135 150 0

@#$#@#$#@
NetLogo 2.0.0
@#$#@#$#@
setup repeat 4 [ step ]
@#$#@#$#@
@#$#@#$#@
